package TumIQTest;

import java.util.ArrayList;
import java.util.List;

import org.jivesoftware.smack.packet.PacketExtension;
import org.jivesoftware.smack.provider.PacketExtensionProvider;
import org.jivesoftware.smackx.FormField;
import org.xmlpull.v1.XmlPullParser;

public class DataFormProviderEx implements PacketExtensionProvider
{
    public static String ELEMENT = "x";
    public static String NAMESPACE = "jabber:x:data";
    
    /**
     * Creates a new DataFormProvider.
     * ProviderManager requires that every PacketExtensionProvider has a public, no-argument constructor
     */
    public DataFormProviderEx() {
    }

    public PacketExtension parseExtension(XmlPullParser parser) throws Exception {
        boolean done = false;
        DataFormEx dataForm = new DataFormEx(parser.getAttributeValue("", "type"));
        while (!done) {
            int eventType = parser.next();
            if (eventType == XmlPullParser.START_TAG) {
                if (parser.getName().equals("instructions")) { 
                    dataForm.addInstruction(parser.nextText());
                }
                else if (parser.getName().equals("title")) {                    
                    dataForm.setTitle(parser.nextText());
                }
                else if (parser.getName().equals("field")) {                    
                    dataForm.addField(parseField(parser));
                }
                // add parse query packet
                else if (parser.getName().equals("query") && parser.getNamespace().equals("jabber:iq:roster")) {
                    // handle items
                    dataForm.setItems(parseRoasterItem(parser));
                }
                /*
                else if (parser.getName().equals("item")) {                    
                    dataForm.addItem(parseItem(parser));
                }
                */
                else if (parser.getName().equals("reported")) {                    
                    dataForm.setReportedData(parseReported(parser));
                }
            } else if (eventType == XmlPullParser.END_TAG) {
                if (parser.getName().equals(dataForm.getElementName())) {
                    done = true;
                }
            }
        }
        return dataForm;
    }
    
    private FormField parseField(XmlPullParser parser) throws Exception {
        boolean done = false;
        FormField formField = new FormField(parser.getAttributeValue("", "var"));
        formField.setLabel(parser.getAttributeValue("", "label"));
        formField.setType(parser.getAttributeValue("", "type"));
        while (!done) {
            int eventType = parser.next();
            if (eventType == XmlPullParser.START_TAG) {
                if (parser.getName().equals("desc")) { 
                    formField.setDescription(parser.nextText());
                }
                else if (parser.getName().equals("value")) {                    
                    formField.addValue(parser.nextText());
                }
                else if (parser.getName().equals("required")) {                    
                    formField.setRequired(true);
                }
                else if (parser.getName().equals("option")) {                    
                    formField.addOption(parseOption(parser));
                }
            } else if (eventType == XmlPullParser.END_TAG) {
                if (parser.getName().equals("field")) {
                    done = true;
                }
            }
        }
        return formField;
    }

    private DataFormEx.ReportedData parseReported(XmlPullParser parser) throws Exception {
        boolean done = false;
        List<FormField> fields = new ArrayList<FormField>();
        while (!done) {
            int eventType = parser.next();
            if (eventType == XmlPullParser.START_TAG) {
                if (parser.getName().equals("field")) { 
                    fields.add(parseField(parser));
                }
            } else if (eventType == XmlPullParser.END_TAG) {
                if (parser.getName().equals("reported")) {
                    done = true;
                }
            }
        }
        return new DataFormEx.ReportedData(fields);
    }

    private FormField.Option parseOption(XmlPullParser parser) throws Exception {
        boolean done = false;
        FormField.Option option = null;
        String label = parser.getAttributeValue("", "label");
        while (!done) {
            int eventType = parser.next();
            if (eventType == XmlPullParser.START_TAG) {
                if (parser.getName().equals("value")) {
                    option = new FormField.Option(label, parser.nextText());                     
                }
            } else if (eventType == XmlPullParser.END_TAG) {
                if (parser.getName().equals("option")) {
                    done = true;
                }
            }
        }
        return option;
    }
    
    /*
    private DataForm.Item parseItem(XmlPullParser parser) throws Exception {
        boolean done = false;
        List<FormField> fields = new ArrayList<FormField>();
        while (!done) {
            int eventType = parser.next();
            if (eventType == XmlPullParser.START_TAG) {
                if (parser.getName().equals("item")) { 
                    fields.add(parseField(parser));
                }
            } else if (eventType == XmlPullParser.END_TAG) {
                if (parser.getName().equals("item")) {
                    done = true;
                }
            }
        }
        return new DataForm.Item(fields);
    }
    */
    
    
    private ArrayList<RoasterItemEx> parseRoasterItem(XmlPullParser parser) throws Exception {
        ArrayList<RoasterItemEx> items = new ArrayList<RoasterItemEx>();
        
        boolean done = false;
        
        String jid = "";
        String name = "";
        String subscription = "";
        String ask = "";
        ArrayList<String> groupsNames = new ArrayList<String>();
        while (!done) {
            int eventType = parser.next();
            if (eventType == XmlPullParser.START_TAG) {
                if (parser.getName().equals("item")) { 
                    // Reset this variable since they are optional for each item
                    groupsNames = new ArrayList<String>();
                    // Initialize the variables from the parsed XML
                    jid = parser.getAttributeValue("", "jid");
                    name = parser.getAttributeValue("", "name");
                    ask = parser.getAttributeValue("", "ask");
                    subscription = parser.getAttributeValue("", "subscription");
                }
                
                if (parser.getName().equals("group")) {
                    groupsNames.add(parser.nextText());
                }
            } else if (eventType == XmlPullParser.END_TAG) {
                if (parser.getName().equals("item")) {
                    System.out.println("****************** One item read over! ******************");
                    System.out.println("Item->jid = " + jid);
                    System.out.println("Item->name = " + name);
                    System.out.println("Item->ask = " + ask);
                    System.out.println("Item->subscription = " + subscription);
                    System.out.println("Item->groupname = " + groupsNames);
                    
                    RoasterItemEx tmpItem = new RoasterItemEx(jid, name, subscription, ask, groupsNames);
                    items.add(tmpItem);
                }
                
                if (parser.getName().equals("query")) {
                    done = true;
                }
            }
        }
        
        return items;
    }
}
